@@ -94,13 +94,13 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
 {
   static GtkTreeView *tree_view;
   GtkTreeModel *model;
-  GtkTreeIter iter;
+  static GtkTreeIter iter;
   static gint column_count = 0;
   static gint row_count = 0;
   static gint n_columns;
   static gboolean editable;
   static gboolean toggles;
-  static gboolean first_time = FALSE;
+  static gboolean first_time = TRUE;
 
   tree_view = GTK_TREE_VIEW (data);
   n_columns = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tree_view), "n_columns"));
@@ -109,8 +109,8 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
 
   model = gtk_tree_view_get_model (tree_view);
 
-  if (!first_time) {
-    first_time = TRUE;
+  if (first_time) {
+    first_time = FALSE;
     gtk_list_store_append (GTK_LIST_STORE (model), &iter);
   }
 
@@ -159,22 +159,22 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
         gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, zenity_util_strip_newline (string->str), -1);        
       }
 
-    if (editable) {
-      gtk_list_store_set (GTK_LIST_STORE (model), &iter, n_columns, TRUE, -1);
-    }
+      if (editable) {
+        gtk_list_store_set (GTK_LIST_STORE (model), &iter, n_columns, TRUE, -1);
+      }
 
-    if (row_count == MAX_ELEMENTS_BEFORE_SCROLLING) {
-      GtkWidget *scrolled_window;
-      GtkRequisition rectangle;
+      if (row_count == MAX_ELEMENTS_BEFORE_SCROLLING) {
+        GtkWidget *scrolled_window;
+        GtkRequisition rectangle;
 
-      gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
-      scrolled_window = glade_xml_get_widget (glade_dialog, "zenity_tree_window");
-      gtk_widget_set_size_request (scrolled_window, -1, rectangle.height);
-      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-                                      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    }
+        gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
+        scrolled_window = glade_xml_get_widget (glade_dialog, "zenity_tree_window");
+        gtk_widget_set_size_request (scrolled_window, -1, rectangle.height);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+                                        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+      }
 
-    column_count ++;
+      column_count++;
 
     } while (g_io_channel_get_buffer_condition (channel) == G_IO_IN); 
     g_string_free (string, TRUE);
@@ -195,9 +195,9 @@ zenity_tree_fill_entries_from_stdin (GtkTreeView  *tree_view,
 {
   GIOChannel *channel;
 
-  g_object_set_data (G_OBJECT (tree_view), "n_columns", (gint *) n_columns);
-  g_object_set_data (G_OBJECT (tree_view), "toggles", (gint *) toggles);
-  g_object_set_data (G_OBJECT (tree_view), "editable", (gint *) editable); 
+  g_object_set_data (G_OBJECT (tree_view), "n_columns", GINT_TO_POINTER (n_columns));
+  g_object_set_data (G_OBJECT (tree_view), "toggles", GINT_TO_POINTER (toggles));
+  g_object_set_data (G_OBJECT (tree_view), "editable", GINT_TO_POINTER (editable)); 
 
   channel = g_io_channel_unix_new (0);
   g_io_channel_set_encoding (channel, NULL, NULL);
@@ -218,7 +218,7 @@ zenity_tree_fill_entries (GtkTreeView  *tree_view,
 
   model = gtk_tree_view_get_model (tree_view);
 
-  g_object_set_data (G_OBJECT (tree_view), "n_columns", (gint *) n_columns);
+  g_object_set_data (G_OBJECT (tree_view), "n_columns", GINT_TO_POINTER (n_columns));
 
   while (args[i] != NULL) {
     gint j;
@@ -403,7 +403,7 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
                                 
         if (tree_data->radiobox) {
           g_object_set (G_OBJECT (cell_renderer), "radio", TRUE, NULL);
-          g_object_set_data (G_OBJECT (model), "radio", (gint *) 1);
+          g_object_set_data (G_OBJECT (model), "radio", GINT_TO_POINTER (1));
         }
 
         g_signal_connect (cell_renderer, "toggled",
@@ -421,7 +421,7 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
           g_signal_connect (G_OBJECT (cell_renderer), "edited",
                             G_CALLBACK (zenity_cell_edited_callback),
           gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view)));
-          g_object_set_data (G_OBJECT (cell_renderer), "column", (gint *) column_index);
+          g_object_set_data (G_OBJECT (cell_renderer), "column", GINT_TO_POINTER (column_index));
 
           column = gtk_tree_view_column_new_with_attributes (tmp->data, 
                                                              cell_renderer, 
@@ -452,7 +452,7 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
         g_signal_connect (G_OBJECT (cell_renderer), "edited",
                           G_CALLBACK (zenity_cell_edited_callback),
         gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view)));
-        g_object_set_data (G_OBJECT (cell_renderer), "column", (gint *) column_index);
+        g_object_set_data (G_OBJECT (cell_renderer), "column", GINT_TO_POINTER (column_index));
 
         column = gtk_tree_view_column_new_with_attributes (tmp->data, 
                                                            cell_renderer, 
@@ -671,7 +671,7 @@ zenity_tree_extract_column_indexes (char *indexes, int n_columns)
     if (index > 0 && index <= n_columns) {
       result[j] = index;
       j++;
-      g_renew (gint, result, j + 1);
+      result = g_renew (gint, result, j + 1);
     }
   }
   result[j] = 0;
